// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang <zhiwei.zhang@saturn.uni-freiburg.de>

#include <assert.h>
#include <iostream>
#include <string>
#include <fstream>
#include <limits>
#include <vector>
#include "./RealTestClass.h"
#include "./source/Other/EliasDeltaEncodingPointer.h"
#include "./source/Other/EliasGammaEncodingPointer.h"
#include "./source/Other/GolombEncodingPointer.h"
#include "./source/Simple8b/Simple8bCode.h"
#include "./source/Simple8b/Simple8bCodeOld.h"
#include "./source/Simple8b/Simple8bCodeNew.h"
#include "./source/Other/VariableByteEncodingPointer.h"

using std::string;
using std::ifstream;
using std::vector;

// _____________________________________________________________________________
RealTestClass::RealTestClass(const string& fileName,
                                  const string& filePath) {
  _filePath = filePath;
  _namesofFiles.clear();
  ifstream fin(fileName.c_str());
  string line;
  while (std::getline(fin, line)) {
    _namesofFiles.push_back(line);
  }
  fin.close();
}

// _____________________________________________________________________________
void RealTestClass::testWithPointerS8() {
  for (auto file : _namesofFiles) {
    for (size_t n = 0; n < 3; ++n) {
      string filename = _filePath + file;
      ifstream fin(filename);
      vector<size_t> vec;
      vec.reserve(1000000000);
      while (fin) {
        size_t idata;
        fin >> idata;
        vec.push_back(idata);
      }
      size_t i = vec.size();
      size_t* plaintext = new size_t[i];
      for (size_t m = 0; m < i; ++m) {
        plaintext[m] = vec[m];
      }
      uint64_t* encoded = new uint64_t[i];
      size_t* decoded = new size_t[i + 239];
      ad_utility::Simple8bCode::encode(plaintext, i, encoded, true);
      ad_utility::Simple8bCode::decode(encoded, i, decoded, true);
      for (size_t j = 0; j < i; ++j) {
        assert(plaintext[j] == decoded[j]);
      }
      fin.close();
      delete[] plaintext;
      delete[] encoded;
      delete[] decoded;
    }
  }
}

// _____________________________________________________________________________
void RealTestClass::testWithPointerS8Old() {
  for (auto file : _namesofFiles) {
    for (size_t n = 0; n < 3; ++n) {
      string filename = _filePath + file;
      ifstream fin(filename);
      vector<size_t> vec;
      vec.reserve(1000000000);
      while (fin) {
        size_t idata;
        fin >> idata;
        vec.push_back(idata);
      }
      size_t i = vec.size();
      size_t* plaintext = new size_t[i];
      for (size_t m = 0; m < i; ++m) {
        plaintext[m] = vec[m];
      }
      uint64_t* encoded = new uint64_t[i];
      size_t* decoded = new size_t[i + 239];
      ad_utility_old::Simple8bCode::encode(plaintext, i, encoded, true);
      ad_utility_old::Simple8bCode::decode(encoded, i, decoded, true);
      for (size_t j = 0; j < i; ++j) {
        assert(plaintext[j] == decoded[j]);
      }
      fin.close();
      delete[] plaintext;
      delete[] encoded;
      delete[] decoded;
    }
  }
}

// _____________________________________________________________________________
void RealTestClass::testWithPointerS8N1() {
  for (auto file : _namesofFiles) {
    for (size_t n = 0; n < 3; ++n) {
      string filename = _filePath + file;
      ifstream fin(filename);
      vector<size_t> vec;
      vec.reserve(1000000000);
      while (fin) {
        size_t idata;
        fin >> idata;
        vec.push_back(idata);
      }
      size_t i = vec.size();
      size_t* plaintext = new size_t[i];
      for (size_t m = 0; m < i; ++m) {
        plaintext[m] = vec[m];
      }
      uint64_t* encoded = new uint64_t[i];
      size_t* decoded = new size_t[i + 239];
      ad_utility_new::Simple8bCodeNew1::encode(plaintext, i, encoded, true);
      ad_utility_new::Simple8bCodeNew1::decode(encoded, i, decoded, true);
      for (size_t j = 0; j < i; ++j) {
        assert(plaintext[j] == decoded[j]);
      }
      fin.close();
      delete[] plaintext;
      delete[] encoded;
      delete[] decoded;
    }
  }
}

// _____________________________________________________________________________
void RealTestClass::testWithPointerS8N2() {
  for (auto file : _namesofFiles) {
    for (size_t n = 0; n < 3; ++n) {
      string filename = _filePath + file;
      ifstream fin(filename);
      vector<size_t> vec;
      vec.reserve(1000000000);
      while (fin) {
        size_t idata;
        fin >> idata;
        vec.push_back(idata);
      }
      size_t i = vec.size();
      size_t* plaintext = new size_t[i];
      for (size_t m = 0; m < i; ++m) {
        plaintext[m] = vec[m];
      }
      uint64_t* encoded = new uint64_t[i];
      size_t* decoded = new size_t[i + 239];
      ad_utility_new::Simple8bCodeNew2::encode(plaintext, i, encoded, true);
      ad_utility_new::Simple8bCodeNew2::decode(encoded, i, decoded, true);
      for (size_t j = 0; j < i; ++j) {
        assert(plaintext[j] == decoded[j]);
      }
      fin.close();
      delete[] plaintext;
      delete[] encoded;
      delete[] decoded;
    }
  }
}


// _____________________________________________________________________________
void RealTestClass::testWithPointerVB() {
  for (auto file : _namesofFiles) {
    for (size_t n = 0; n < 3; ++n) {
      string filename = _filePath + file;
      ifstream fin(filename);
      vector<size_t> vec;
      vec.reserve(1000000000);
      while (fin) {
        size_t idata;
        fin >> idata;
        vec.push_back(idata);
      }
      size_t i = vec.size();
      size_t* plaintext = new size_t[i];
      for (size_t m = 0; m < i; ++m) {
        plaintext[m] = vec[m];
      }
      char* encoded = new char[i * 5];
      size_t* decoded = new size_t[i];
      VariableByteEncodingPointer::encode(plaintext, i, encoded, true);
      VariableByteEncodingPointer::decode(encoded, i, decoded, true);
      for (size_t j = 0; j < i; ++j) {
        assert(plaintext[j] == decoded[j]);
      }
      fin.close();
      delete[] plaintext;
      delete[] encoded;
      delete[] decoded;
    }
  }
}
