// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang <zhiwei.zhang@saturn.uni-freiburg.de>

#ifndef ZHIWEI_ZHANG_SOURCE_OTHER_GOLOMBENCODINGPOINTER_H_
#define ZHIWEI_ZHANG_SOURCE_OTHER_GOLOMBENCODINGPOINTER_H_

#include <math.h>
#include <iostream>
#include "../Timer.h"

class GolombEncodingPointer {
 public:

  // Encodes the gaps in @list via Golomb encoding using the mod parameter @M.
  template<typename Numeric>
  static void encode(Numeric* plaintext, const int& M, const size_t& nofElements,
                       bool* encoded, const bool& show = false) {
    Timer t;
    t.start();
    size_t k = 0;
    int nofRemainderBits = ceil(log2(M)) - 1;
    for (size_t i = 0; i < nofElements; ++i) {
      int q = plaintext[i] / M;
      Numeric r = plaintext[i] - q * M;
      for (int j = 0; j < q; ++j) encoded[k++] = 0;
      encoded[k++] = 1;
      for (int j = nofRemainderBits; j >= 0; --j) {
        encoded[k++] = ((r & (1 << j)) != 0);
      }
    }
    t.stop();
    if (show) {
      std::cout << nofElements << "\t" << k / 8 << "\t"
      << t.getUSecs() << "\t";
    }
  }

  // Assumes @bits contains the gaps of a Doclist encoded with golomb M=@M,
  // Restores the Doclist.
  template<typename Numeric>
  static void decode(const bool* bits, const int& M, const size_t& nofElements,
              Numeric* decoded, const bool& show = false) {
    Timer t;
    t.start();
    int nofRemainderBits = ceil(log2(M)) - 1;
    size_t i = 0;
    size_t k = 0;
    while (k < nofElements) {
      Numeric lastId = 0;
      size_t q = i;
      while (bits[i] == 0) { ++i; }
      q = i - q;
      ++i;
      Numeric r = 0;
      for (int j = nofRemainderBits; j >= 0; --j) {
        if (bits[i++] == 1) r += (1 << j);
      }
      lastId += q * M + r;
      decoded[k++] = lastId;
    }
    t.stop();
    if (show) std::cout << t.getUSecs() << std::endl;
  }
};

#endif  // ZHIWEI_ZHANG_SOURCE_OTHER_GOLOMBENCODINGPOINTER_H_
