// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang <zhiwei.zhang@saturn.uni-freiburg.de>

#ifndef ZHIWEI_ZHANG_SOURCE_OTHER_VARIABLEBYTEENCODINGPOINTER_H_
#define ZHIWEI_ZHANG_SOURCE_OTHER_VARIABLEBYTEENCODINGPOINTER_H_

#include <stdint.h>
#include <stddef.h>
#include <vector>
#include <string>
#include <iostream>
#include <bitset>
#include "../Timer.h"

using std::vector;
using std::bitset;

class VariableByteEncodingPointer {
  public:

  // Encodes the gaps in @list via vbyte encoding.
  template<typename Numeric>
  static void encode(Numeric* plaintext, const size_t& nofElements,
                       char* encoded, const bool& show = false) {
    Timer t;
    t.start();
    int k = 0;
    for (size_t i = 0; i < nofElements; ++i) {
      int x = plaintext[i];
      do {
        int mod = x % 128;
        x = x / 128;
        encoded[k++] = (x == 0 ? mod : mod + 128);
      } while (x > 0);
    }
    t.stop();
    if (show) {
      std::cout << nofElements << "\t" << k << "\t"
      << t.getUSecs() << "\t";
    }
  }

  // Assumes @bits contains the gaps of a Doclist encoded with vbyte,
  // Restores the Doclist.
  template<typename Numeric>
  static void decode(char* bytes, const size_t& nofElements,
     Numeric* decoded, const bool& show = false) {
    Timer t;
    t.start();
    size_t k = 0;
    size_t m = 0;
    while (m < nofElements) {
      int x = 0;
      int f = 1;
      while (bytes[k] < 0) {
        x += f * (bytes[k++] + 128);
        f *= 128;
      }
      x += f * bytes[k++];
      decoded[m++] = x;
    }
    t.stop();
    if (show) std::cout << t.getUSecs() << std::endl;
  }

  // Helper for tests, etc.
  static std::string byteVectorToString(const char* bytes, const size_t& nofElements) {
    std::ostringstream os;
    for (size_t i = 0; i < nofElements; ++i) {
      bitset<8> x(bytes[i]);
      os << x;
      if (i < nofElements - 1) os << ' ';
    }
    return os.str();
  }
};

#endif  // ZHIWEI_ZHANG_SOURCE_OTHER_VARIABLEBYTEENCODINGPOINTER_H_
