// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang

#include <gtest/gtest.h>
#include <string>
#include "../source/Other/EliasGammaEncodingPointer.h"

using std::string;
using std::to_string;

// _____________________________________________________________________________
TEST(EliasGammaEncodingPointerTest, encode) {
  int* list = new int[2];
  bool* result1 = new bool[1];
  bool* result2 = new bool[15];
  bool* result3 = new bool[16];
  string resultToString;

  list[0] = 1;
  EliasGammaEncodingPointer::encode(list, 1, result1);
  resultToString += to_string(result1[0]);
  ASSERT_EQ("1", resultToString);


  resultToString.clear();
  list[0] = 150;
  // as elias gamma: 00000 00100 10110
  EliasGammaEncodingPointer::encode(list, 1, result2);
  for (int i = 0; i < 15; i++) {
    resultToString += to_string(result2[i]);
  }
  ASSERT_EQ("000000010010110", resultToString);


  resultToString.clear();
  list[0] = 1;
  list[1] = 150;
  // Gaps 1 and 150 like before
  EliasGammaEncodingPointer::encode(list, 2, result3);
  for (int i = 0; i < 16; i++) {
    resultToString += to_string(result3[i]);
  }
  ASSERT_EQ("1000000010010110", resultToString);

  delete[] list;
  delete[] result1;
  delete[] result2;
  delete[] result3;
}

// _____________________________________________________________________________
TEST(EliasGammaEncodingPointerTest, encodeDecode) {
  int* plaintext = new int[5];
  int* decoded = new int[5];
  bool* encoded = new bool[1000];

  plaintext[0] = 1;
  plaintext[1] = 5;
  plaintext[2] = 16400;
  plaintext[3] = 20000;
  plaintext[4] = 70000;

  EliasGammaEncodingPointer::encode(plaintext, 5, encoded);
  EliasGammaEncodingPointer::decode(encoded, 5, decoded);
  for (size_t i = 0; i < 5; ++i) {
    ASSERT_EQ(plaintext[i], decoded[i]);
  }

  delete[] plaintext;
  delete[] decoded;
  delete[] encoded;
}
