// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang

#include <gtest/gtest.h>
#include <string>
#include "../source/Other/GolombEncodingPointer.h"

using std::string;
using std::to_string;

// _____________________________________________________________________________
TEST(GolombEncodingPointerTest, encode) {
  int* list = new int[2];
  int M = 16;
  bool* result1 = new bool[5];
  bool* result2 = new bool[14];
  bool* result3 = new bool[19];
  string resultToString;

  list[0] = 1;
  GolombEncodingPointer::encode(list, M, 1, result1);
  for (int i = 0; i < 5; i++) {
    resultToString += to_string(result1[i]);
  }
  ASSERT_EQ("10001", resultToString);


  resultToString.clear();
  list[0] = 150;
  // as golomb: 00000000010110
  GolombEncodingPointer::encode(list, M, 1, result2);
  for (int i = 0; i < 14; i++) {
    resultToString += to_string(result2[i]);
  }
  ASSERT_EQ("00000000010110", resultToString);


  resultToString.clear();
  list[0] = 1;
  list[1] = 150;
  // Gaps 1 and 150 like before
  GolombEncodingPointer::encode(list, M, 2, result3);
  for (int i = 0; i < 19; i++) {
    resultToString += to_string(result3[i]);
  }
  ASSERT_EQ("1000100000000010110", resultToString);

  delete[] list;
  delete[] result1;
  delete[] result2;
  delete[] result3;
}

// _____________________________________________________________________________
TEST(GolombEncodingPointerTest, encodeDecode) {
  int* plaintext = new int[5];
  int M = 16;
  int* decoded = new int[5];
  bool* encoded = new bool[5000];

  plaintext[0] = 1;
  plaintext[1] = 5;
  plaintext[2] = 16400;
  plaintext[3] = 20000;
  plaintext[4] = 70000;

  GolombEncodingPointer::encode(plaintext, M, 5, encoded);
  GolombEncodingPointer::decode(encoded, M, 5, decoded);
  for (size_t i = 0; i < 5; ++i) {
    ASSERT_EQ(plaintext[i], decoded[i]);
  }

  delete[] plaintext;
  delete[] decoded;
  delete[] encoded;
}
