// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang
#include <random>
#include <math.h>
#include <gtest/gtest.h>
#include "../source/Simple8b/Simple8bCodeNew.h"
#include "../source/GenerateNumber.h"

// _____________________________________________________________________________
TEST(Simple8bCodeNewTest, New1Decode) {

  {
    int* plaintext = new int[60];
    uint64_t* encoded = new uint64_t[60];
    int* decoded = new int[60 + 239];
    for (size_t i = 0; i < 60; ++i) {
      plaintext[i] = pow(i, 2);
    }

    ad_utility_new::Simple8bCodeNew1::encode(plaintext, 60, encoded);
    ad_utility_new::Simple8bCodeNew1::decode(encoded, 60, decoded);
    for (size_t i = 0; i < 60; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }

    delete[] plaintext;
    delete[] encoded;
    delete[] decoded;
  }

  {
    int* plaintext = new int[1000];
    uint64_t* encoded = new uint64_t[1000];
    int* decoded = new int[1000 + 239];
    GenerateNumber::uniformDistribution(1, 100, 1000, plaintext);

    ad_utility_new::Simple8bCodeNew1::encode(plaintext, 1000, encoded);
    ad_utility_new::Simple8bCodeNew1::decode(encoded, 1000, decoded);
    for (size_t i = 0; i < 1000; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }

    delete[] plaintext;
    delete[] encoded;
    delete[] decoded;
  }

  {
    int* plaintext = new int[1000];
    uint64_t* encoded = new uint64_t[1000];
    int* decoded = new int[1000 + 239];
    GenerateNumber::geometricDistribution(0.3, 1000, plaintext);

    ad_utility_new::Simple8bCodeNew1::encode(plaintext, 1000, encoded);
    ad_utility_new::Simple8bCodeNew1::decode(encoded, 1000, decoded);
    for (size_t i = 0; i < 1000; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }

    delete[] plaintext;
    delete[] encoded;
    delete[] decoded;
  }
}

// _____________________________________________________________________________
TEST(Simple8bCodeNewTest, New2Decode) {

  {
    int* plaintext = new int[60];
    uint64_t* encoded = new uint64_t[60];
    int* decoded = new int[60 + 239];
    for (size_t i = 0; i < 60; ++i) {
      plaintext[i] = pow(i, 2);
    }

    ad_utility_new::Simple8bCodeNew2::encode(plaintext, 60, encoded);
    ad_utility_new::Simple8bCodeNew2::decode(encoded, 60, decoded);
    for (size_t i = 0; i < 60; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }

    delete[] plaintext;
    delete[] encoded;
    delete[] decoded;
  }

  {
    int* plaintext = new int[1000];
    uint64_t* encoded = new uint64_t[1000];
    int* decoded = new int[1000 + 239];
    GenerateNumber::uniformDistribution(1, 100, 1000, plaintext);

    ad_utility_new::Simple8bCodeNew2::encode(plaintext, 1000, encoded);
    ad_utility_new::Simple8bCodeNew2::decode(encoded, 1000, decoded);
    for (size_t i = 0; i < 1000; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }

    delete[] plaintext;
    delete[] encoded;
    delete[] decoded;
  }

  {
    int* plaintext = new int[1000];
    uint64_t* encoded = new uint64_t[1000];
    int* decoded = new int[1000 + 239];
    GenerateNumber::geometricDistribution(0.3, 1000, plaintext);

    ad_utility_new::Simple8bCodeNew2::encode(plaintext, 1000, encoded);
    ad_utility_new::Simple8bCodeNew2::decode(encoded, 1000, decoded);
    for (size_t i = 0; i < 1000; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }

    delete[] plaintext;
    delete[] encoded;
    delete[] decoded;
  }
}
