// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang
#include <random>
#include <math.h>
#include <vector>
#include <gtest/gtest.h>
#include "../source/Simple8b/Simple8bCodeVector.h"
#include "../source/GenerateNumber.h"

using std::vector;

// _____________________________________________________________________________
TEST(Simple8bCodeVectorTest, decode) {

  vector<int> plaintext;
  vector<uint64_t> encoded;
  vector<int> decoded;

  {
    plaintext.reserve(60);
    for (size_t i = 0; i < 60; ++i) {
      plaintext.push_back(pow(i, 2));
    }
    encoded.reserve(60);
    decoded.reserve(60);

    ad_utility_vector::Simple8bCodeVector::encode(plaintext, 60, encoded);
    ad_utility_vector::Simple8bCodeVector::decode(encoded, 60, decoded);
    for (size_t i = 0; i < 60; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }
  }

  plaintext.clear();
  encoded.clear();
  decoded.clear();

  {
    plaintext.reserve(1000);
    GenerateNumber::uniformDistribution(1, 100, 1000, plaintext);
    encoded.reserve(1000);
    decoded.reserve(1000);

    ad_utility_vector::Simple8bCodeVector::encode(plaintext, 1000, encoded);
    ad_utility_vector::Simple8bCodeVector::decode(encoded, 1000, decoded);
    for (size_t i = 0; i < 1000; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }
  }

  plaintext.clear();
  encoded.clear();
  decoded.clear();

  {
    plaintext.reserve(1000);
    GenerateNumber::geometricDistribution(0.3, 1000, plaintext);
    encoded.reserve(1000);
    decoded.reserve(1000);

    ad_utility_vector::Simple8bCodeVector::encode(plaintext, 1000, encoded);
    ad_utility_vector::Simple8bCodeVector::decode(encoded, 1000, decoded);
    for (size_t i = 0; i < 1000; ++i) {
      ASSERT_EQ(plaintext[i], decoded[i]);
    }
  }
}
